package de.ullisroboterseite.UrsAI2WakeLock;

// Autor: http://UllisRoboterSeite.de

// Doku:  http://UllisRoboterSeite.de/android-AI2-WakeLock.html
// Created: 2020-04-01
//
// Version 1 (2020-04-01)
// -------------------------
// - Basis-Version
//
// Version 1.1 (2020-07-16)
// -------------------------
// - RequestIgnoreBatteryOptimization hinzugefügt
// - IsIgnoringBatteryOptimizations hinzugefügt
// - OpenBatteryOptimizationSettings hinzugefügt
//
//
// Version 1.2 (2020-08-05)
// -------------------------
// - Release hatte keine Wirkung
//
//
// Version 1.3 (2020-08-08)
// -------------------------
// - Release hat die interne Referenz nicht zurückgesetzt.
//
//
// Version 1.4 (2021-01-26)
// -------------------------
// - WiFi Lock hinzugefügt.
//
// Version 1.5 (2021-03-21)
// -------------------------
// - Eigenschaften Version und VersionSDK hinzugefügt.
// - SDK-Versionsprügung bei Funktionen zur Einstellung der Akku-Optimierung
// - Fehler bei RequestIgnoreBatteryOptimization behoben
//
// Version 1.6 (2021-04-09)
// -------------------------
// - IsRunningInCompanion hinzugefügt.
//
// Version 1.7 (2021-07-20)
// -------------------------
// - AquireFullWakelock, OnAfterRelease, AcquireCausesWakeup, EnableShowOnLockScreen hinzugefügt.

import java.util.*;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;

import android.content.Context;
import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.PowerManager;
import android.os.Build;
import android.view.WindowManager;
import android.app.KeyguardManager;
import android.app.KeyguardManager.*;
import android.provider.Settings;
import android.net.wifi.WifiManager;

@DesignerComponent(version = 1, //
        versionName = UrsAI2WakeLock.VersionName, //
        dateBuilt = UrsAI2WakeLock.dateBuilt, //
        description = "AI2 extension block for establishing an Android WakeLock.", //
        category = com.google.appinventor.components.common.ComponentCategory.EXTENSION, //
        nonVisible = true, //
        helpUrl = "http://UllisRoboterSeite.de/android-AI2-WakeLock.html", //
        iconName = "aiwebres/icon.png")
@UsesPermissions(permissionNames = "android.permission.WAKE_LOCK,android.permission.REQUEST_IGNORE_BATTERY_OPTIMIZATIONS")
@SimpleObject(external = true)
public class UrsAI2WakeLock extends AndroidNonvisibleComponent implements Component {
    static final String LOG_TAG = "KEEPALIVE";
    static final String VersionName = "1.7.0";
    static final String dateBuilt = "2021-07-20";
    final Context thisContext;
    final Activity thisActivity;
    final Form thisForm;
    final UrsAI2WakeLock thisInstance = this;
    PowerManager.WakeLock wakeLock = null;
    int wakeLockType = 0;

    WifiManager.WifiLock wifiLock = null;

    boolean onAfterRelease = false;
    boolean acquireCausesWakeup = false;

    PowerManager powerManager;

    public UrsAI2WakeLock(ComponentContainer container) {
        super(container.$form());
        thisActivity = (Activity) container.$context();
        thisContext = container.$context();
        thisForm = container.$form();
        powerManager = (PowerManager) thisContext.getSystemService(Context.POWER_SERVICE);
    } // ctor

    @SimpleProperty(description = "Returns the component's version name.")
    public String Version() {
        return VersionName;
    }

    @SimpleProperty(description = "Returns running Android SDK version.")
    public int VersionSDK() {
        return Build.VERSION.SDK_INT;
    }

    @SimpleProperty(description = "Returns wether the app is running in the companion.")
    public boolean IsRunningInCompanion() {
        return form instanceof ReplForm;
    }

    @SimpleProperty(description = "True if WakeLock is established.")
    public boolean IsActive() {
        return (wakeLock != null);
    }

    // https://developer.android.com/reference/android/os/PowerManager#ON_AFTER_RELEASE
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
    @SimpleProperty(description = "When this wake lock is released, poke the user activity timer so the screen stays on for a little longer.")
    public void OnAfterRelease(boolean value) {
        onAfterRelease = value;
    }

    @SimpleProperty(description = "When this wake lock is released, poke the user activity timer so the screen stays on for a little longer.")
    public boolean OnAfterRelease() {
        return onAfterRelease;
    }

    // https://developer.android.com/reference/android/os/PowerManager#ACQUIRE_CAUSES_WAKEUP
    @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
    @SimpleProperty(description = "Turn the screen on when the wake lock is acquired.")
    public void AcquireCausesWakeup(boolean value) {
        acquireCausesWakeup = value;
    }

    @SimpleProperty(description = "Turn the screen on when the wake lock is acquired.")
    public boolean AcquireCausesWakeup() {
        return acquireCausesWakeup;
    }

    @SimpleProperty(description = "Type of established WakeLock: 0: none, 1: PARTIAL_WAKE_LOCK, "
            + "2: SCREEN_DIM_WAKE_LOCK, 3: SCREEN_BRIGHT_WAKE_LOCK, 4: FULL_WAKE_LOCK")
    public int WakeLockType() {
        return wakeLockType;
    }

    int getFlags(int level) {
        if (level != PowerManager.PARTIAL_WAKE_LOCK) {
            if (onAfterRelease)
                level |= PowerManager.ON_AFTER_RELEASE;
            if (acquireCausesWakeup)
                level |= PowerManager.ACQUIRE_CAUSES_WAKEUP;
        }
        return level;
    }

    @SimpleFunction(description = "Ensures that the screen is on at full brightness.")
    public void AquireScreenBrightWakeLock() {
        if (wakeLock != null)
            Release();
        wakeLock = powerManager.newWakeLock(getFlags(PowerManager.SCREEN_BRIGHT_WAKE_LOCK), "URS::WakeLock");
        wakeLock.acquire();
        wakeLockType = 3;
    }

    @SimpleFunction(description = "Ensures that the screen is on (but may be dimmed).")
    public void AquireScreenDimWakeLock() {
        if (wakeLock != null)
            Release();
        wakeLock = powerManager.newWakeLock(getFlags(PowerManager.SCREEN_DIM_WAKE_LOCK), "URS::WakeLock");
        wakeLock.acquire();
        wakeLockType = 2;
    }

    @SimpleFunction(description = "Ensures that the CPU is running; the screen will be allowed to go off.")
    public void AquirePartialWakeLock() {
        if (wakeLock != null)
            Release();
        wakeLock = powerManager.newWakeLock(getFlags(PowerManager.PARTIAL_WAKE_LOCK), "URS::WakeLock");
        wakeLock.acquire();
        wakeLockType = 1;
    }

    @SimpleFunction(description = "Ensures that the CPU is running; the screen will be allowed to go off.")
    public void AquireFullWakeLock() {
        if (wakeLock != null)
            Release();
        wakeLock = powerManager.newWakeLock(getFlags(PowerManager.FULL_WAKE_LOCK), "URS::WakeLock");
        wakeLock.acquire();
        wakeLockType = 4;
    }

    @SimpleFunction(description = "Release WakeLock.")
    public void Release() {
        wakeLockType = 0;
        if (wakeLock == null)
            return;
        wakeLock.release();
        wakeLock = null;
    }

    @SimpleFunction(description = "Request Ignore Battery Optimization.")
    public void RequestIgnoreBatteryOptimization() {
        if (Build.VERSION.SDK_INT < 23)
            return;
        Intent intent = new Intent(Settings.ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS);
        String packageName = "package:" + thisContext.getPackageName();
        intent.setData(Uri.parse(packageName));
        thisActivity.startActivity(intent);
    }

    @SimpleFunction(description = "Show screen for controlling which apps can ignore battery optimizations")
    public void OpenBatteryOptimizationSettings() {
        if (Build.VERSION.SDK_INT < 23)
            return;
        Intent intent = new Intent();
        intent.setAction(Settings.ACTION_IGNORE_BATTERY_OPTIMIZATION_SETTINGS);
        thisContext.startActivity(intent);
    }

    @SimpleProperty(description = "Returns whether the given application package name is on the device's power whitelist.")
    public boolean IsIgnoringBatteryOptimizations() {
        if (Build.VERSION.SDK_INT < 23)
            return false;
        String packageName = thisContext.getPackageName();
        return powerManager.isIgnoringBatteryOptimizations(packageName);
    }

    @SimpleFunction(description = "Ensures that the WiFi keeps alive.")
    public void AquireWifiLock() {
        if (wifiLock == null) {
            WifiManager wm = (WifiManager) thisContext.getSystemService(Context.WIFI_SERVICE);
            wifiLock = wm.createWifiLock(WifiManager.WIFI_MODE_FULL_HIGH_PERF, "UrsWifiLock");
        }
        if (wifiLock.isHeld())
            return;
        wifiLock.acquire();
    }

    @SimpleFunction(description = "Releases the WiFi lock.")
    public void ReleaseWifiLock() {
        if (wifiLock == null)
            return;
        if (!wifiLock.isHeld())
            return;
        wifiLock.release();
    }

    // https://codedocu.de/Google/Android/Anleitungen/Android_colon_-mit-to-open-an-alarm-an-activity-even-in-locked-screen?3195
    // https://stackoverflow.com/questions/15393689/unlock-screen-programmatically-and-show-activity
    // https://developer.android.com/reference/android/app/KeyguardManager#requestDismissKeyguard(android.app.Activity,%20android.app.KeyguardManager.KeyguardDismissCallback)
    @SimpleFunction(description = "Shows window on LockScreen")
    public void EnableShowOnLockScreen() {
        if (Build.VERSION.SDK_INT >= 27) {
            KeyguardManager keyguardManager = (KeyguardManager) form.getSystemService(Context.KEYGUARD_SERVICE);
            form.setShowWhenLocked(true);
            form.setTurnScreenOn(true);
            keyguardManager.requestDismissKeyguard(form, null);
        }
        form.getWindow().addFlags(WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD
                | WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
                | WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

    }
}